# -*- coding: utf-8; Mode: Python; indent-tabs-mode: nil; tab-width: 4 -*-

# Written by Kobe Lee.
#
# Plugin test

import os
import subprocess

from ubiquity import plugin
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GLib
import firstboot_utils as ff

NAME = '30s'
AFTER = 'trustinfo'
WEIGHT = 10
INSTALL = False
# edit by kobe
# OEM = False

class PageGtk(plugin.PluginUI):
    plugin_title = 'ubiquity/text/30s_heading_label'

    def __init__(self, controller, *args, **kwargs):
        plugin.PluginUI.__init__(self)

        self.controller = controller
        builder = Gtk.Builder()
        builder.add_from_file(os.path.join(os.environ['UBIQUITY_GLADE'], 'step30S.ui'))
        builder.connect_signals(self)
        self.controller.add_builder(builder)
        self.page = builder.get_object('step30S')

        self.module_id_entry = builder.get_object('module_id_entry')
        self.check_module_id_entry = builder.get_object('check_module_id_entry')
        self.username_entry = builder.get_object('name_entry')
        self.module_pw_entry = builder.get_object('module_pw_entry')
        self.ukey_pw_entry = builder.get_object('ukey_pw_entry')

        self.username_entry.set_sensitive(False)

        self.check_module_id_tip = builder.get_object('check_module_id_tip')
        self.module_id_tip = builder.get_object('module_id_tip')

        self.module_id_entry.identity = 'module_id_entry'
        self.module_id_entry.connect('changed', self.on_text_changed)
        self.module_id_entry.connect('focus-out-event', self.on_focus_out)

        self.check_module_id_entry.identity = 'check_module_id_entry'
        self.check_module_id_entry.connect('changed', self.on_text_changed)
        self.check_module_id_entry.connect('focus-out-event', self.on_focus_out)
        self.check_module_id_entry.connect('key-press-event', self.disable_paste, None)
        self.check_module_id_entry.connect('button-press-event', self.disable_paste, None)

        # 1 for blank content, 2 for length error, 0 for success
        self.module_pw_reason = 1
        self.ukey_pw_reason = 1
        self.module_pw_entry.connect('changed', self.on_module_pw_changed)
        self.ukey_pw_entry.connect('changed', self.on_ukey_pw_changed)

        self.error_label = builder.get_object('error_label')
        self.error_label.set_use_markup(True)
        self.error_label.set_markup('')

        self.tip_label = builder.get_object('tip_label')
        self.tip_label.set_use_markup(True)
        self.tip_label.set_markup('<span>即将进行可信密码模块资源导入，请勿拔出管理UKEY。</span>')

        self.plugin_widgets = self.page

    def disable_interaction(self):
        self.module_id_entry.set_sensitive(False)
        self.check_module_id_entry.set_sensitive(False)
        self.module_pw_entry.set_sensitive(False)
        self.ukey_pw_entry.set_sensitive(False)

    def enable_interaction(self):
        self.module_id_entry.set_sensitive(True)
        self.check_module_id_entry.set_sensitive(True)
        self.module_pw_entry.set_sensitive(True)
        self.ukey_pw_entry.set_sensitive(True)

    def disable_paste(self, widget, event, data):
        if event.type == Gdk.EventType.BUTTON_PRESS and \
           event.button == 2:
            return True
    
        if event.type == Gdk.EventType.KEY_PRESS and \
           event.state & Gdk.ModifierType.CONTROL_MASK and \
           event.keyval == Gdk.KEY_v:
            return True
    
        return False

    def check_id(self):
        module_id = self.module_id_entry.get_text().strip()
        check_module_id = self.check_module_id_entry.get_text().strip()

        if not module_id:
            self.module_id_tip.set_markup('<span color="darkred">模块ID不能为空</span>')
        else:
            self.module_id_tip.set_markup('')

        if check_module_id == '':
            self.check_module_id_tip.set_markup('')
            return
            
        matched = False
        if module_id != check_module_id:
            self.check_module_id_tip.set_markup('<span color="darkred">两次输入的模块ID不一致</span>')
        else:
            self.check_module_id_tip.set_markup('')
            matched = True

        if matched and module_id:
            self._find_username(module_id)
        else:
            self.username_entry.set_text('')

    def on_text_changed(self, widget):
        if widget.identity == 'check_module_id_entry':
            self.check_id()
            return

        self.check_module_id_tip.set_markup('')
        self.username_entry.set_text('')

        module_id = self.module_id_entry.get_text().strip()
        if not module_id:
            self.module_id_tip.set_markup('<span color="darkred">模块ID不能为空</span>')
        else:
            self.module_id_tip.set_markup('')

    def on_focus_out(self, widget, event):
        self.check_id()
        return False

    def _find_username(self, module_id):
        if not os.path.isdir('/mnt/ICFile'):
            self.username_entry.set_text('')
            return False

        files = os.listdir('/mnt/ICFile')
        for f in files:
            if not f.startswith('sys_') or not f.endswith('.dat'):
                continue
            sub = f[4: -4]
            parts = sub.split('_')
            username = parts[-1]
            try:
                f_id = '%s_%s' % (parts[0], parts[1])
            except IndexError:
                f_id = ''
            if f_id == module_id:
                self.username_entry.set_text(username)
                break

        if self.username_entry.get_text() == '':
            self.module_id_tip.set_markup('<span color="darkred">模块ID输入错误</span>')
        else:
            self.module_id_tip.set_markup('')

    def on_module_pw_changed(self, widget):
        module_pw = self.module_pw_entry.get_text().strip()
        if not module_pw:
            self.error_label.set_markup('<span color="darkred">模块密码不能为空</span>')
            self.module_pw_reason = 1
        elif len(module_pw) < 8 or len(module_pw) > 16:
            self.error_label.set_markup('<span color="darkred">模块密码长度必须为&lt;8, 16&gt;位</span>')
            self.module_pw_reason = 2
        else:
            self.error_label.set_markup('')
            self.module_pw_reason = 0

    def on_ukey_pw_changed(self, widget):
        ukey_pw = self.ukey_pw_entry.get_text().strip()
        if not ukey_pw:
            self.error_label.set_markup('<span color="darkred">UKEY密码不能为空</span>')
            self.ukey_pw_reason = 1
        elif len(ukey_pw) < 6 or len(ukey_pw) > 16:
            self.error_label.set_markup('<span color="darkred">UKEY密码长度必须为&lt;6, 16&gt;位</span>')
            self.ukey_pw_reason = 2            
        else:
            self.error_label.set_markup('')
            self.ukey_pw_reason = 0            


class Page(plugin.Plugin):
    def prepare(self, unfiltered=False):
        self.ui.error_label.set_markup('')
        return plugin.Plugin.prepare(self, unfiltered)

    # kobe
    def update_kti_info(self, succeed, username, module_pw):
        if not succeed:
            try:
                subprocess.call(['rm', '-f', '/etc/kti/user.info'])
            except Exception:
                pass
            return

        if not username or not module_pw:
            return

        if not os.path.isdir('/etc/kti'):
            try:
                subprocess.call(['rm', '-rf', '/etc/kti'])
            except Exception:
                pass
            try:
                subprocess.call(['mkdir', '-p', '/etc/kti'])
            except Exception:
                pass

        f = open('/etc/kti/user.info', 'wb')
        f.write(bytes('%s\n' % username, 'UTF-8'))
        f.write(bytes('%s\n' % module_pw, 'UTF-8'))
        f.close()

    def check_tcm_call(self):
        if self.ellipsis == '...':
            self.ellipsis = '.'
        elif self.ellipsis == '..':
            self.ellipsis = '...'
        elif self.ellipsis == '.':
            self.ellipsis = '..'

        if self.tcm.poll() is None:
            self.ui.error_label.set_markup('<span color="darkred">正在导入密码资源，请稍后(约25秒)%s</span>' % self.ellipsis)
            return True

        tcm_sysinfo_ret = self.tcm.returncode

        if tcm_sysinfo_ret:
            tcm_sysinfo_ret -= 256

        err_message = None

        if tcm_sysinfo_ret == 0:
            self.ui.error_label.set_markup('')
            ff.showErrorMessage('密码资源导入成功')
            self.ui.enable_interaction()
            plugin.Plugin.ok_handler(self)
            return False
        elif tcm_sysinfo_ret == -1:
            err_message = '获取Ukey中IC卡映像文件失败'
        elif tcm_sysinfo_ret == -2:
            err_message = '可信密码模块ID与用户名不匹配'
            self.ui.module_id_entry.grab_focus()
        elif tcm_sysinfo_ret == -3:
            err_message = '密码资源导入失败'
        elif tcm_sysinfo_ret == -4:
            err_message = '连接Ukey失败'
        elif tcm_sysinfo_ret == -5:
            err_message = '登录Ukey失败'
        elif tcm_sysinfo_ret == -6:
            err_message = '动态库加载失败'
        elif tcm_sysinfo_ret == -7:
            err_message = '用户名或口令输入错误'
        elif tcm_sysinfo_ret == -8:
            err_message = '获取可信密码模块公钥失败'
        elif tcm_sysinfo_ret == -9:
            err_message = 'Ukey 加密转换错误'
        elif tcm_sysinfo_ret == -10:
            err_message = '输入参数错误'
        else:
            err_message = '未知错误，请联系管理中心'

        self.update_kti_info(False, None, None)
        self.ui.error_label.set_markup('<span color="darkred">%s</span>' % err_message)
        self.ui.enable_interaction()
        self._stopForward()
        return False

    def ok_handler(self):

        ukey = ff.findDevice()
        if not ukey:
            self.ui.error_label.set_markup('<span color="darkred">请插入UKEY或确认UKEY已插好，并重试</span>')
            self._stopForward()
            return

        if not ff.mountDevice(ukey):
            self.ui.error_label.set_markup('<span color="darkred">UKEY挂载错误</span>')
            self._stopForward()
            return

        self.ui.check_id()

        if self.ui.username_entry.get_text() == '':
            if self.ui.check_module_id_tip.get_text() != '':
                self.ui.check_module_id_entry.grab_focus()
            if self.ui.module_id_tip.get_text() != '':
                self.ui.module_id_entry.grab_focus()
            self.ui.error_label.set_markup('')
            self._stopForward()
            return

        if self.ui.module_pw_reason != 0 or self.ui.ukey_pw_reason != 0:
            if self.ui.module_pw_reason == 1:
                self.ui.error_label.set_markup('<span color="darkred">模块密码不能为空</span>')
                self.ui.module_pw_entry.grab_focus()
            elif self.ui.module_pw_reason == 2:
                self.ui.error_label.set_markup('<span color="darkred">模块密码长度必须为&lt;8, 16&gt;位</span>')
                self.ui.module_pw_entry.grab_focus()
            elif self.ui.ukey_pw_reason == 1:
                self.ui.error_label.set_markup('<span color="darkred">UKEY密码不能为空</span>')
                self.ui.ukey_pw_entry.grab_focus()
            elif self.ui.ukey_pw_reason == 2:
                self.ui.error_label.set_markup('<span color="darkred">UKEY密码长度必须为&lt;6, 16&gt;位</span>')
                self.ui.ukey_pw_entry.grab_focus()

            self._stopForward()
            return
        
        module_id = self.ui.module_id_entry.get_text().strip()
        username = self.ui.username_entry.get_text().strip()
        module_pw = self.ui.module_pw_entry.get_text().strip()
        ukey_pw = self.ui.ukey_pw_entry.get_text().strip()

        if not os.path.isfile("/usr/tss/tcm_sysinfo"):
            self.ui.error_label.set_markup('<span color="darkred">/usr/tss/tcm_sysinfo文件不存在</span>')
            self._stopForward()
            return

        self.ellipsis = '...'
        self.ui.error_label.set_markup('<span color="darkred">正在导入密码资源，请稍后(约25秒)%s</span>' % self.ellipsis)
        try:
            self.tcm = subprocess.Popen(['/usr/tss/tcm_sysinfo', module_id, username, module_pw, ukey_pw])
            self.update_kti_info(True, username, module_pw)
            GLib.timeout_add(1000, self.check_tcm_call)
        except Exception:
            self.ui.error_label.set_markup('<span color="darkred">调用tcm_sysinfo出错</span>')
            self.tcm = None
            self._stopForward()
            return

        self.ui.disable_interaction()
        self.done = False
        self.succeeded = False

    def _stopForward(self):
        self.enter_ui_loop()
