# -*- coding: utf-8; Mode: Python; indent-tabs-mode: nil; tab-width: 4 -*-

# Written by Kobe Lee.
#
# Plugin test

import os
import subprocess

from ubiquity import plugin
from gi.repository import Gtk
import firstboot_utils as ff

NAME = 'trustinfo'
#AFTER = 'usersetup'
AFTER = 'welcome'
WEIGHT = 10
INSTALL = False
# edit by kobe
# OEM = False

class PageGtk(plugin.PluginUI):
    plugin_title = 'ubiquity/text/trustinfo_heading_label'

    def __init__(self, controller, *args, **kwargs):
        plugin.PluginUI.__init__(self)
        self.controller = controller
        builder = Gtk.Builder()
        builder.add_from_file(os.path.join(os.environ['UBIQUITY_GLADE'], 'stepTrustInfo.ui'))
        builder.connect_signals(self)
        self.controller.add_builder(builder)
        self.page = builder.get_object('stepKylin')
        self.label = builder.get_object('label1')
        self.label.set_markup('<span>系统就绪前还需要几个步骤，请确定已经插入管理UKEY</span>')

        if not os.path.exists('/etc/os_safe.d'):
            subprocess.call(['mkdir', '/etc/os_safe.d', '-p'])
        else:
            if os.path.isfile('/etc/os_safe.d') or os.path.islink('/etc/os_safe.d'):
                subprocess.call(['rm', '/etc/os_safe.d'])
                subprocess.call(['mkdir', '/etc/os_safe.d', '-p'])

        self.error_label = builder.get_object('error_label')
        self.error_label.set_use_markup(True)
        self.error_label.set_markup('')

        self.plugin_widgets = self.page

class Page(plugin.Plugin):
    def prepare(self, unfiltered=False):
        self.ui.error_label.set_markup('')
        return plugin.Plugin.prepare(self, unfiltered)

    def _stopForward(self):
        self.done = False
        self.enter_ui_loop()

    def ok_handler(self):
        self.ui.error_label.set_markup('')
        ukey = ff.findDevice()

        if not ukey:
            self.ui.error_label.set_markup('<span color="darkred">请插入UKEY或确认UKEY已插好，并重试</span>')
            self._stopForward()
            return

        if not ff.mountDevice(ukey):
            self.ui.error_label.set_markup('<span color="darkred">UKEY挂载错误</span>')
            self._stopForward()
            return

        if not os.path.isfile("/usr/bin/ht_regcheck"):
            self.ui.error_label.set_markup('<span color="darkred">ht_regcheck不存在</span>')
            self._stopForward()
            return

        ret_regcheck = None
        try:
            ret_regcheck = subprocess.call(['/usr/bin/ht_regcheck'])
        except Exception:
            self.ui.error_label.set_markup('<span color="darkred">调用ht_regcheck时出错</span>')
            self._stopForward()
            return

        if ret_regcheck:
            ret_regcheck -= 256

        if ret_regcheck == -2:
            dlg = Gtk.MessageDialog(None, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.YES_NO,
                                    '系统将设置为可信软件发布服务器，请确认后点击继续。')
            dlg.set_position(Gtk.WindowPosition.CENTER)
            dlg.set_modal(True)
            rc = dlg.run()
            dlg.destroy()
            if rc == Gtk.ResponseType.NO:
                self._stopForward()
                return 
            elif rc == Gtk.ResponseType.YES:
                plugin.Plugin.ok_handler(self)
                return 

        if ret_regcheck == 0:
            dlg = Gtk.MessageDialog(None, 0, Gtk.MessageType.INFO, Gtk.ButtonsType.YES_NO,
                                    '系统将设置为普通终端，请确认后点击继续。')
            dlg.set_position(Gtk.WindowPosition.CENTER)
            dlg.set_modal(True)
            rc = dlg.run()
            dlg.destroy()
            if rc == Gtk.ResponseType.NO:
                self._stopForward()
                return 
            elif rc == Gtk.ResponseType.YES:
                plugin.Plugin.ok_handler(self)
                return 

        if ret_regcheck == -1:
            self.ui.error_label.set_markup('<span color="darkred">UKEY中未找到配置文件，请联系管理中心进行配置文件导入</span>')
            self._stopForward()
            return

        if ret_regcheck == -3:
            self.ui.error_label.set_markup('<span color="darkred">证书完整性被破坏，请到管理系统或发布终端重新导入证书到UKEY</span>')
            self._stopForward()
            return

        self.ui.error_label.set_markup('<span color="darkred">未知错误，请重试或联系管理中心</span>')
        self._stopForward()
